// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2010 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
package de.uka.ilkd.key.rtsj.proof.init;

import java.util.HashMap;

import de.uka.ilkd.key.gui.IMain;
import de.uka.ilkd.key.gui.POBrowser;
import de.uka.ilkd.key.gui.configuration.ChoiceSettings;
import de.uka.ilkd.key.gui.configuration.ProofSettings;
import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.proof.init.JavaProfile;
import de.uka.ilkd.key.proof.init.ProblemInitializer;
import de.uka.ilkd.key.proof.init.proofobligation.DefaultPOProvider;
import de.uka.ilkd.key.rtsj.proof.init.proofobligation.RTSJPOProvider;
import de.uka.ilkd.key.rtsj.rule.UseOperationContractRuleRTSJ;
import de.uka.ilkd.key.rule.AbstractUseOperationContractRule;

public class RTSJProfile extends JavaProfile {

    public static final Name HEAP_SPACE_NAME = new Name("heapSpace");
	/**
	 *  determines whether memory consumption is taken into account in proofs or not
	 */
	private boolean memory = false;
	
    public RTSJProfile(boolean memory) {
        this(null);
        this.memory = memory;
    }
    
    public RTSJProfile(IMain main) {
        super("standardRules-RTSJ.key", main);
    }
    
    public void updateSettings(ProofSettings settings) {        
        ChoiceSettings cs = settings.getChoiceSettings();
        HashMap<String, String> dcs = cs.getDefaultChoices();
        dcs.put("rtsj", "rtsj:on");
        if(memory){
	    dcs.put("memory", "memory:on");
        }else{
	    dcs.put("memory", "memory:off");
	}
        cs.setDefaultChoices(dcs);
    }
    
    
    /**
     * the name of the profile
     */
    public String name() {
        return "Realtime Java (RTSJ) Profile";
    }
    
    public boolean memoryConsumption(){
    	return memory;
    }
    
    public AbstractUseOperationContractRule getContractRule() {
        return memory ? UseOperationContractRuleRTSJ.INSTANCE_RTSJ_MEM :
            UseOperationContractRuleRTSJ.INSTANCE_RTSJ_MEM;
    }
    
    /**
     * returns the file name of the internal class directory relative to JavaRedux
     * @return the file name of the internal class directory relative to JavaRedux
     */
    public String getInternalClassDirectory() {
	return "rtsjperc";
    }
    
    /**
     * returns the file name of the internal class list
     * @return the file name of the internal class list
     */
    public String getInternalClasslistFilename() {
	 return "JAVALANGRTSJ.TXT";
    }
    
    public DefaultPOProvider getPOProvider() {
	return new RTSJPOProvider(memory);
    }
    
    public Class<? extends POBrowser> getPOBrowserClass() {
	return de.uka.ilkd.key.rtsj.gui.POBrowserRTSJ.class;
    }

    public ProblemInitializer createProblemInitializer(IMain main) {
	 return new RTSJProblemInitializer(main);
    }
    
}
