// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2010 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
package de.uka.ilkd.key.rtsj.rule.metaconstruct;

import java.util.Iterator;

import de.uka.ilkd.key.java.ProgramElement;
import de.uka.ilkd.key.java.Services;
import de.uka.ilkd.key.java.abstraction.ArrayType;
import de.uka.ilkd.key.java.expression.literal.IntLiteral;
import de.uka.ilkd.key.java.expression.operator.NewArray;
import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.Namespace;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.TermFactory;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.logic.sort.ProgramSVSort;
import de.uka.ilkd.key.rule.inst.SVInstantiations;

/**
 * Creates a function term arraySize(sizeOf(type),d_1,...,d_n) from an expression 
 * new type[d_1,...,d_n].
 */
public class ArraySize extends AbstractMetaOperator {

  
    public ArraySize() {
        super(new Name("#arraySize"), 1);
    }

    /**
     * Creates a function term arraySize_type(d_1,...,d_n) from an expression 
     * new type[d_1,...,d_n].
     */
    public Term calculate(Term term, SVInstantiations svInst, Services services) {
//        ProgramVariable v0 = (ProgramVariable) term.sub(0).op();
        Iterator<SchemaVariable> it = svInst.svIterator();
        TermFactory tf = TermFactory.DEFAULT;
        NewArray na=null;
        while(it.hasNext()){
            SchemaVariable sv = it.next();
            if(((SchemaVariableAdapter) sv).sort() == ProgramSVSort.NEWARRAY){
                na = (NewArray) svInst.getInstantiation(sv);          
            }
        }
        if(na.getArrayInitializer()!=null){
            return term.sub(0);
        }
//        System.out.println(na.getArguments().getExpression(0).getClass());
        Term dim=null;

        if(ProgramSVSort.SIMPLEEXPRESSION.canStandFor((ProgramElement) na.getArguments().get(0), 
                svInst.getExecutionContext(), services)){
            dim = services.getTypeConverter().convertToLogicElement(
                    na.getArguments().get(0), svInst.getExecutionContext());
        }else{
            return term.sub(0);
        }
        Namespace funcs = services.getNamespaces().functions();
        
        ProgramVariable consumed = services.getJavaInfo().getAttribute("consumed", 
                services.getJavaInfo().getKeYJavaTypeByClassName("javax.realtime.MemoryArea"));
        Term heapSpaceTerm = tf.createAttributeTerm(consumed, 
        	services.getTypeConverter().convertToLogicElement(svInst.getExecutionContext().getMemoryAreaAsRef(), 
        		svInst.getExecutionContext()));
        Function sizeFunc;
        String baseType = ((ArrayType) na.getKeYJavaType().getJavaType()).
            getBaseType().getKeYJavaType().getSort().toString();
        Term entrySize = null;
        sizeFunc = (Function) funcs.lookup(new Name("arraySize"));
        if(na.getDimensions()>1){
            IntLiteral lit = new IntLiteral("4");
            entrySize = services.getTypeConverter().convertToLogicElement(lit);
        }else if(baseType.equals("jbyte") || baseType.equals("boolean")){
            IntLiteral lit = new IntLiteral("1");
            entrySize = services.getTypeConverter().convertToLogicElement(lit);
        }else if(baseType.equals("jshort") || baseType.equals("jchar")){
            IntLiteral lit = new IntLiteral("2");
            entrySize = services.getTypeConverter().convertToLogicElement(lit);
        }else if(baseType.equals("jlong")){
            IntLiteral lit = new IntLiteral("8");
            entrySize = services.getTypeConverter().convertToLogicElement(lit);
        }else{
            IntLiteral lit = new IntLiteral("4");
            entrySize = services.getTypeConverter().convertToLogicElement(lit);
        }
        Term size = tf.createFunctionTerm(sizeFunc, entrySize, dim);
        return tf.createUpdateTerm(heapSpaceTerm,
                tf.createFunctionTerm((TermSymbol) funcs.lookup(new Name("add")),
                        heapSpaceTerm, size), term.sub(0));
    }

}
